/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef TArmeTLC_h
#include "TArmeTLC.h"
#endif //TArmeTLC_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

TArmeTLC::TArmeTLC(const char* name, TArmeDoc* doc) :
	TArmeContainer(name,doc)
{
}

TArmeTLC::TArmeTLC(ArmeArchiveHandler* handler) : 
	TArmeContainer(handler)
{
}

bool TArmeTLC::WriteToArchive(ArmeArchiveHandler* handler) const
{
	return TArmeContainer::WriteToArchive(handler);
}

SInt32 TArmeTLC::GetType() const
{
	return kTLC;
}

void TArmeTLC::PosWillChange()
{
	//nothing to do
}
void TArmeTLC::PosHasChanged()
{
	//nothing to do
}

/*
void TArmeTLC::AddOurStereoAudioData(float** buffers, int level, SInt64 start, SInt64 length)
{
	ZzAssert(0);
}

void TArmeTLC::CopyOurStereoAudioData(float** buffers, int level, SInt64 start, SInt64 length)
{
	ZzAssert(0);
}

void TArmeTLC::AddOurStereoAudioDataForVAO(
		SInt32	vao, 
		float	leftFactor, 
		float	rightFactor, 
		float**	buffers, 
		int 	level, 
		SInt64 	start, 
		SInt64 	length)
{
	// make sure the hierarchy is not too deep
	ZzAssert(level < kMaxHierarchyDepth);
	
	SInt32 nbFrames = (SInt32)(length);

	TArmeWSObj* child;
	
	SInt32 nbChildrenWithVAO = GetChildrenCountWithVAO(vao);
	
	bool bAtLeastAChildIntersects = false;
	if(nbChildrenWithVAO > 0)
	{
		child = GetChildWithVAO(vao,0);
		SInt64 childStartTime = GET_VAL(child->GetStartTime());
	
#define READ_FRAMES_QUICK_N_DIRTY
//#define READ_FRAMES_INTELLIGENT
//#define READ_FRAMES_WRONG

#ifdef READ_FRAMES_INTELLIGENT
		SInt64 bufferEndxTime = start + length;
		SInt64 childEndxTime = childStartTime + GET_VAL(child->GetDuration());
		// do they intersect ?
		if( ! (( (childStartTime - child->GetPrerollNbFrames()) >= bufferEndxTime) || (start >= childEndxTime)) )
		{
			bAtLeastAChildIntersects = true;
			child->CopyOurStereoAudioData(buffers, level+1, start - childStartTime, length);
		}
#elif defined READ_FRAMES_QUICK_N_DIRTY
		bAtLeastAChildIntersects = true;
		child->CopyOurStereoAudioData(buffers, level+1, start - childStartTime, length);
#elif defined READ_FRAMES_WRONG
		SInt64 bufferEndxTime = start + length;
		SInt64 childEndxTime = childStartTime + GET_VAL(child->GetDuration());
		if( ! ((childStartTime >= bufferEndxTime) || (start >= childEndxTime)) )
		{
			bAtLeastAChildIntersects = true;
			child->CopyOurStereoAudioData(buffers, level+1, start - childStartTime, length);
		}
#endif
	}
	if(nbChildrenWithVAO > 1)
	{
		// ask our other children to mix their in the level+1 buffer
		for( int i = 1; i < GetChildrenCountWithVAO(vao); i++ )
		{
			child = GetChildWithVAO(vao,i);
			SInt64 childStartTime = GET_VAL(child->GetStartTime());
			
#ifdef READ_FRAMES_INTELLIGENT
			SInt64 childEndxTime = childStartTime + GET_VAL(child->GetDuration());
			SInt64 bufferEndxTime = start + length;
			if( ! (((childStartTime - child->GetPrerollNbFrames()) >= bufferEndxTime) || (start >= childEndxTime)) )
			{
				bAtLeastAChildIntersects = true;
				child->AddOurStereoAudioData(buffers, level+1, start - childStartTime, length);
			}
#elif defined READ_FRAMES_QUICK_N_DIRTY
			bAtLeastAChildIntersects = true;
			child->AddOurStereoAudioData(buffers, level+1, start - childStartTime, length);
#elif defined READ_FRAMES_WRONG
			SInt64 childEndxTime = childStartTime + GET_VAL(child->GetDuration());
			SInt64 bufferEndxTime = start + length;
			if( ! ((childStartTime >= bufferEndxTime) || (start >= childEndxTime)) )
			{
				bAtLeastAChildIntersects = true;
				child->AddOurStereoAudioData(buffers, level+1, start - childStartTime, length);
			}
#endif
		}
	}
		
	float leftMax = 0.0;
	float rightMax = 0.0;
	if(bAtLeastAChildIntersects)
	{
		float* src = buffers[level+1];
		float* dst = buffers[level];
		float leftSrc,rightSrc;
		float leftPow,rightPow;
			
		for( SInt32 frame = 0; frame < nbFrames; ++frame )
		{
			leftSrc = leftFactor * *src++;
			rightSrc = rightFactor * *src++;
			*dst++ +=  leftSrc;
			*dst++ +=  rightSrc;
			leftPow = leftSrc*leftSrc;
			rightPow = rightSrc*rightSrc; 
			if(leftPow > leftMax)
				leftMax = leftPow;
			if(rightPow > rightMax)
				rightMax = rightPow;
		}
	}
	fDoc->UpdateVAOLeftLevelWithValue(vao,leftMax);
	fDoc->UpdateVAORightLevelWithValue(vao,rightMax);
	// otherwise, we don't add anything the buffer. that's ok since it's zeroed at start
}


*/